###############################################################################
#
#  main.py --
#
#  $LastChangedDate$
#  $LastChangedRevision$
#  $LastChangedBy$
#
#  Author: ltomaso
#
#  History: 
#    March 5, 2013 - initial revision
#    May 13, 2013 - modification in InputParametersEdited method (Now, the minimum value attribute is equal to 3*entity.NumberOfInputParametersDefined) 
#
###############################################################################

# load Ans.DesignXplorer.CSharpOptimizerExample.dll
def onInit(context):
    if context == "DesignXplorer":    
        dlldir = System.IO.Path.Combine(ExtAPI.ExtensionMgr.CurrentExtension.InstallDir,System.IO.Path.Combine("bin",System.IO.Path.Combine("Win64","Release")))
        dllpath = System.IO.Path.Combine(dlldir,"Ans.DesignXplorer.CSharpOptimizerExample.dll")
        clr.AddReferenceToFileAndPath(dllpath)
    
# creates the optimizer
def createOptimizer(entity):
    from Ansys.DesignXplorer.CSharpOptimizerExample import CSharpOptimizer
    return CSharpOptimizer()
    
# returns true when all optimization settings are defined and correct and the optimizer is ready to run
def canRunOptimizer(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value
    
    # checks the consistency between the number of sample points and the number of candidate points
    if maximumNumberOfCandidates <= 0 or maximumNumberOfCandidates > numberOfSamples:
         return False
        
    # numberOfSamples must be positive and less than 10000
    if numberOfSamples <= 0 or numberOfSamples > 10000:
         return False

    # at least one objective or one constraint must be defined on an input or output parameter
    if entity.NumberOfObjectivesDefined == 0 and entity.NumberOfConstraintsDefined == 0 and entity.NumberOfObjectivesDefinedOnInputs == 0 and entity.NumberOfConstraintsDefinedOnInputs == 0:
        return False

    # the optimization can be run
    return True

# returns a brief description of the optimization algorithm
def getDescription(entity):
    description = "Random screening - C# Example"
    return description

# returns a brief description of the 2nd optimization algorithm (limited to 10 continuous input parameters)
def getDescription2(entity):
    description = "Random screening - C# Example 2 (limited to 10 continuous input parameters)"
    return description

# returns a brief description of the optimization configuration
def getConfiguration(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value
    configuration = 'Generates %d random points, and extracts %d candidates.' %(numberOfSamples,maximumNumberOfCandidates)
    return configuration

# returns a quick help message when canRunOptimizer returns false
def getQuickHelp(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value

    if numberOfSamples <= 0 or numberOfSamples > 10000:
        return 'Number of Samples must be positive and lower than 10000'

    if maximumNumberOfCandidates <= 0 or maximumNumberOfCandidates > numberOfSamples:
        return 'Maximum Number of Candidates must be positive and lower than Number of Samples'

    if entity.NumberOfObjectivesDefined == 0 and entity.NumberOfConstraintsDefined == 0 and entity.NumberOfObjectivesDefinedOnInputs == 0 and entity.NumberOfConstraintsDefinedOnInputs == 0:
        return 'At least one objective or constraint needs to be defined'

    return ''

# a possible use of this callback is to change attributes or the visibility of a property
# in this example, we have changed the minimum value attribute
def InputParametersEdited(entity):
    # the minimum number of sample points must be equal to or greater than 3 * the number of input parameters
    entity.Properties["MyNumberOfSamples"].Attributes["min"]=3*entity.NumberOfInputParametersDefined
    
