////////////////////////////////////////////////////////////////////////////
//
// COPYRIGHT ANSYS. ALL RIGHTS RESERVED.
//
// $LastChangedDate$
// $LastChangedRevision$
// $LastChangedBy$
//
// Author: ltomaso
//
// Decription : implementation of the CppOptimizer
//
////////////////////////////////////////////////////////////////////////////

#include "CppOptimizerExample.h"

#include <ctime>
#include <cstdlib>

static MyCppOptimizer _opt;// static optimzer

void MyCppOptimizer::Init()
{
    srand(1);// intialization of random generator
    Clear();
}

void MyCppOptimizer::Clear()
{
    _iNumInputs = 0;
    _iNumOutputs = 0;
    _iNumObjectives = 0;
    _iNumConstraints = 0;
    _iMaxNumCandidates = 0;
    _iMaxNumPoints = 0;
    _bestcostvalue = 0;
    _feasiblecandidate = false;

    _params.clear();
    _inputs.clear();
    _objectives.clear();
    _constraints.clear();
    _pointsToSubmit.clear();
    _candidates.clear();
    _allpoints.clear();
}

CPPOPTIMIZEREXAMPLE_API void AddDoubleVariable(int iId, double dLowerBound, double dUpperBound, double dInitialValue)
{
    if ( iId == 0 )
        _opt.Init();

    MyParameter param;
    param._id = iId;
    param._type = eDouble;
    param._initialvalue = dInitialValue;// not used yet
    param._values.push_back(dLowerBound);
    param._values.push_back(dUpperBound);

    _opt._inputs.push_back(param);
    _opt._iNumInputs++;
}

CPPOPTIMIZEREXAMPLE_API void AddObjective(int iId, int iObjectiveType, double dTargetValue, double dImportance)
{
    MyObjective obj;
    obj._id = iId;
    switch(iObjectiveType)
    {
    case 1:
        obj._type = eMinimize;
        break;
    case 2:
        obj._type = eMaximize;
        break;
    case 3:
        obj._type = eSeekTarget;
        obj._targetvalue = dTargetValue;
        break;
    }

    obj._importance = dImportance;

    _opt._objectives.push_back(obj);
    _opt._iNumObjectives++;
}

CPPOPTIMIZEREXAMPLE_API void AddConstraint(int iId, int iConstraintType, double dConstraintFirstValue, double dConstraintSecondValue, double dImportance)
{
    MyConstraint constr;
    constr._id = iId;
    switch(iConstraintType)
    {
    case 1:
        constr._type = eLessThan;
        constr._constraintFirstValue = dConstraintFirstValue;
        break;
    case 2:
        constr._type = eGreaterThan;
        constr._constraintFirstValue = dConstraintFirstValue;
        break;
    case 3:
        constr._type = eEqualTo;
        constr._constraintFirstValue = dConstraintFirstValue;
        break;
    case 4:
        constr._type = eInsideBounds;
        constr._constraintFirstValue = dConstraintFirstValue;
        constr._constraintSecondValue = dConstraintSecondValue;
        break;
    }

    constr._importance = dImportance;

    _opt._constraints.push_back(constr);
    _opt._iNumConstraints++;
}

CPPOPTIMIZEREXAMPLE_API void PutNumberOfSamples(int iNumSamples)
{
    _opt._iMaxNumPoints = iNumSamples;
}

CPPOPTIMIZEREXAMPLE_API bool CanRun(void)
{
    return ( _opt._iNumInputs > 0 && _opt._iNumOutputs > 0 && _opt._iNumObjectives > 0 && _opt._iMaxNumPoints > 0 );
}

CPPOPTIMIZEREXAMPLE_API void PutMaximumNumberOfCandidates(int iMaxNumCandidates)
{
    _opt._iMaxNumCandidates = iMaxNumCandidates;
}

CPPOPTIMIZEREXAMPLE_API int RunOptimization()
{
    int iNumPoints = (int)_opt._pointsToSubmit.size();
    if ( iNumPoints > 0 )
        return iNumPoints;

    iNumPoints = (int)_opt._allpoints.size();
    if ( iNumPoints >= _opt._iMaxNumPoints )
        return 0;

    MyPoint point;
    std::vector<MyParameter>::iterator iter;
    double drand;
    double dval = 0;
    for ( iter = _opt._inputs.begin(); iter != _opt._inputs.end(); iter++ )
    {
        drand = (double)rand()/RAND_MAX;
        if ( (*iter)._type == eDouble )
        {
            dval = (*iter)._values[0] + drand * ((*iter)._values[1]-(*iter)._values[0]);
            point._inputs.push_back(dval);
        }
    }
    point._submitted = false;
    _opt._pointsToSubmit.push_back(point);
    return 1;
}

CPPOPTIMIZEREXAMPLE_API void GetInputValues(int iSubmittedPointId, double *dValues)
{
    if ( iSubmittedPointId >= _opt._pointsToSubmit.size() )
        return;

    std::vector<double>::iterator iter;
    int ival = 0;
    for ( iter = _opt._pointsToSubmit[iSubmittedPointId]._inputs.begin(); iter != _opt._pointsToSubmit[iSubmittedPointId]._inputs.end(); iter++, ival++ )
    {
        dValues[ival] = (*iter);
    }
    _opt._pointsToSubmit[iSubmittedPointId]._submitted = true;
    _opt._pointsToSubmit[iSubmittedPointId]._uptodate = false;

    return;
}

CPPOPTIMIZEREXAMPLE_API void AddOutputValues(int iSubmittedPointId, bool success, double *dValues, int iNumValues)
{
    if ( iSubmittedPointId >= _opt._pointsToSubmit.size() )
        return;

    if ( success )
    {
        for ( int ival = 0; ival < iNumValues; ival++ )
        {
            _opt._pointsToSubmit[iSubmittedPointId]._outputs.push_back(dValues[ival]);
        }

        std::vector<MyObjective>::iterator iterobj;
        int iobj = 0;
        double dval = 0, dobj = 0;
        _opt._pointsToSubmit[iSubmittedPointId]._costfunction = 0;
        _opt._pointsToSubmit[iSubmittedPointId]._feasible = true;
        for ( iterobj = _opt._objectives.begin(); iterobj != _opt._objectives.end(); iterobj++ )
        {
            iobj = (*iterobj)._id;
            if ( iobj < _opt._iNumInputs )
                dval = _opt._pointsToSubmit[iSubmittedPointId]._inputs[iobj];
            else
                dval = _opt._pointsToSubmit[iSubmittedPointId]._outputs[iobj-_opt._iNumInputs];

            dobj = 0;
            switch ((*iterobj)._type)
            {
            case eMinimize:
                dobj = dval;
                break;
            case eMaximize:
                dobj = -dval;
                break;
            case eSeekTarget:
                dobj = fabs(dval-(*iterobj)._targetvalue);
                break;
            }
            _opt._pointsToSubmit[iSubmittedPointId]._costfunction += dobj;
        }

        std::vector<MyConstraint>::iterator iterconstr;

        int iconstr = 0;
        double dviolation = 0;
        for ( iterconstr = _opt._constraints.begin(); iterconstr != _opt._constraints.end(); iterconstr++ )
        {
            iconstr = (*iterconstr)._id;
            if ( iobj < _opt._iNumInputs )
                dval = _opt._pointsToSubmit[iSubmittedPointId]._inputs[iconstr];
            else
                dval = _opt._pointsToSubmit[iSubmittedPointId]._outputs[iconstr-_opt._iNumInputs];

            dviolation = 0;
            switch((*iterconstr)._type)
            {
            case eLessThan:
                dviolation = dval - (*iterconstr)._constraintFirstValue;
                break;
            case eGreaterThan:
                dviolation = (*iterconstr)._constraintFirstValue - dval;
                break;
            case eEqualTo:
                dviolation = abs((*iterconstr)._constraintFirstValue - dval);
                break;
            case eInsideBounds:
                dviolation = dval - (*iterconstr)._constraintSecondValue;
                if (((*iterconstr)._constraintFirstValue - dval) > dviolation )
                    dviolation = ((*iterconstr)._constraintFirstValue - dval);
                break;
            }
            _opt._pointsToSubmit[iSubmittedPointId]._costfunction += dviolation;
            if ( dviolation > 0 )
                _opt._pointsToSubmit[iSubmittedPointId]._feasible = false;
        }

        if ( _opt._candidates.size() == 0 )
        {
            _opt._bestcostvalue = _opt._pointsToSubmit[iSubmittedPointId]._costfunction;
            _opt._feasiblecandidate = _opt._pointsToSubmit[iSubmittedPointId]._feasible;
            _opt._candidates.push_back(_opt._pointsToSubmit[iSubmittedPointId]);
        }
        else
        {
            int ic = 0;
            bool newcandidate = false;
            std::vector<MyPoint>::iterator iterc;
            for ( iterc = _opt._candidates.begin(); iterc != _opt._candidates.end(); iterc++, ic++ )
            {
                if ( (*iterc)._costfunction > _opt._pointsToSubmit[iSubmittedPointId]._costfunction )
                {
                    if ( ic == 0 )
                    {
                        _opt._bestcostvalue = _opt._pointsToSubmit[iSubmittedPointId]._costfunction;
                        _opt._feasiblecandidate = _opt._pointsToSubmit[iSubmittedPointId]._feasible;
                    }
                    _opt._candidates.insert(iterc,_opt._pointsToSubmit[iSubmittedPointId]);
                    newcandidate = true;
                    break;
                }
            }

            if ( newcandidate )
            {
                if ( (int)_opt._candidates.size() > _opt._iMaxNumCandidates )
                {
                    _opt._candidates.resize(_opt._iMaxNumCandidates);
                }
            }
            else if ( (int)_opt._candidates.size() < _opt._iMaxNumCandidates )
            {
                _opt._candidates.push_back(_opt._pointsToSubmit[iSubmittedPointId]);
            }
        }
    }

    _opt._allpoints.push_back(_opt._pointsToSubmit[iSubmittedPointId]);
    _opt._pointsToSubmit[iSubmittedPointId]._uptodate = true;
    _opt._pointsToSubmit[iSubmittedPointId]._success = success;

    std::vector<MyPoint>::iterator iterpt;
    for ( iterpt = _opt._pointsToSubmit.begin(); iterpt != _opt._pointsToSubmit.end(); iterpt++ )
    {
        if ( !(*iterpt)._uptodate )
            return;
    }

    // all points have been updated;
    _opt._pointsToSubmit.clear();
}

CPPOPTIMIZEREXAMPLE_API int GetNumberOfGeneratedCanidatePoints(void)
{
    return (int) _opt._candidates.size();
}

CPPOPTIMIZEREXAMPLE_API void GetCandidatePointValues(int iCandidatePointId, double *dValues)
{
    if (iCandidatePointId >= (int)_opt._candidates.size())
        return;

    std::vector<double>::iterator iter;
    int ival = 0;
    for ( iter = _opt._candidates[iCandidatePointId]._inputs.begin(); iter != _opt._candidates[iCandidatePointId]._inputs.end(); iter++, ival++ )
    {
        dValues[ival] = (*iter);
    }

    for ( iter = _opt._candidates[iCandidatePointId]._outputs.begin(); iter != _opt._candidates[iCandidatePointId]._outputs.end(); iter++, ival++ )
    {
        dValues[ival] = (*iter);
    }

    return;
}

CPPOPTIMIZEREXAMPLE_API void GetCostFunctionValues(double *cost, bool *feasible)
{
    cost[0] = _opt._bestcostvalue;
    feasible[0] = _opt._feasiblecandidate;
    return;
}

CPPOPTIMIZEREXAMPLE_API wchar_t * GetStatus()
{
    if ((int)_opt._allpoints.size() < _opt._iMaxNumPoints )
        return L"Interrupted";
    return L"Complete";
}

CPPOPTIMIZEREXAMPLE_API void Clear()
{
    _opt.Clear();
}
